#pragma once

#include "note.h"
#include "precisionevaluator.h"
#include "classicactions.h"

#include <memory>
#include <array>

class ClassicSprite;
class ClassicGraphicsManager;
class ClassicAnimationScenario;

class ClassicNote : public Note
{
public:

    enum class Grade
    {
        PERFECT,
           GOOD,
            BAD
    };

    enum State
    {
          NONE,

        FLYING,
        ACTIVE,
         DYING,
          DEAD
    };

    explicit ClassicNote(const std::vector<microsec>& intervals, microsec perfect_offset,
                         Type type, const Coordinates& coord,
                         const std::unique_ptr<ClassicGraphicsManager>& manager);
    virtual ~ClassicNote() = default;

    virtual bool isActive() const override;
    virtual void update(const microsec &music_offset) override;
    virtual void input(PlayerInput&& inputdata) override;
    virtual void putToGame(const microsec &music_offset) override;
    virtual bool isInGame() const override;
    virtual bool isExpired() const override;
    virtual void draw() const override;

    Type type() const noexcept;

    std::shared_ptr<ClassicSprite> sprite() const noexcept;
    void setSprite(const std::shared_ptr<ClassicSprite>& sprite) noexcept;
    const Coordinates& getCoordinates() const noexcept;

private:

    struct NoteElement
    {
        std::shared_ptr<ClassicSprite> _sprite;
        std::array<std::shared_ptr<ClassicAnimationScenario>, 5> _animations;

        const std::array<const sf::Keyboard::Key, 2> _keys;
        const Coordinates _coordinates;
        const Type _type;

        bool pressed = false;
        sf::Keyboard::Key pressed_as;
    };

    std::vector<NoteElement> _elements;

    const PrecisionEvaluator<Grade> _evaluator;
    const std::unique_ptr<ClassicGraphicsManager>& _graphics_manager;

    State _state;
};
